@@ -22,7 +22,7 @@ build_time_vars = {'ABIFLAGS': '',
  'BOOTSTRAP_HEADERS': '\\',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'armv7--netbsdelf-eabihf',
+ 'BUILD_GNU_TYPE': 'armv6--netbsdelf-eabihf',
  'BYTESTR_DEPS': '\\',
  'CC': 'gcc -pthread',
  'CCSHARED': '-fPIC',
@@ -49,14 +49,14 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFIG_ARGS': "'--enable-shared' '--with-openssl=/usr' '--with-system-expat' "
                 "'--with-system-ffi' '--without-ensurepip' "
                 "'--without-pymalloc' '--with-readline=readline' "
-                "'--prefix=/usr/pkg' '--build=armv7--netbsdelf-eabihf' "
-                "'--host=armv7--netbsdelf-eabihf' '--mandir=/usr/pkg/man' "
+                "'--prefix=/usr/pkg' '--build=armv6--netbsdelf-eabihf' "
+                "'--host=armv6--netbsdelf-eabihf' '--mandir=/usr/pkg/man' "
                 "'--enable-option-checking=yes' "
-                "'build_alias=armv7--netbsdelf-eabihf' "
-                "'host_alias=armv7--netbsdelf-eabihf' "
-                "'PKG_CONFIG=/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/.tools/bin/pkg-config' "
+                "'build_alias=armv6--netbsdelf-eabihf' "
+                "'host_alias=armv6--netbsdelf-eabihf' "
+                "'PKG_CONFIG=/pbulk/work/lang/python311/work/.tools/bin/pkg-config' "
                 "'PKG_CONFIG_PATH=' "
-                "'PKG_CONFIG_LIBDIR=/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/.buildlink/lib/pkgconfig:/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/.buildlink/share/pkgconfig' "
+                "'PKG_CONFIG_LIBDIR=/pbulk/work/lang/python311/work/.buildlink/lib/pkgconfig:/pbulk/work/lang/python311/work/.buildlink/share/pkgconfig' "
                 "'CC=gcc' 'CFLAGS=-O2 -I/usr/include -I/usr/pkg/include' "
                 "'LDFLAGS=-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib "
                 "-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib' "
@@ -64,9 +64,9 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '/usr/pkg/include',
  'CONFINCLUDEPY': '/usr/pkg/include/python3.11',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/Python-3.11.12/coverage.info',
+ 'COVERAGE_INFO': '/pbulk/work/lang/python311/work/Python-3.11.13/coverage.info',
  'COVERAGE_LCOV_OPTIONS': '--rc lcov_branch_coverage=1',
- 'COVERAGE_REPORT': '/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/Python-3.11.12/lcov-report',
+ 'COVERAGE_REPORT': '/pbulk/work/lang/python311/work/Python-3.11.13/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--rc lcov_branch_coverage=1 --branch-coverage '
                             '--title "CPython 3.11 LCOV report [commit $(shell '
                             ')]"',
@@ -363,7 +363,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_MADVISE': 1,
  'HAVE_MAKEDEV': 1,
  'HAVE_MBRTOWC': 1,
- 'HAVE_MEMFD_CREATE': 1,
+ 'HAVE_MEMFD_CREATE': 0,
  'HAVE_MEMORY_H': 1,
  'HAVE_MEMRCHR': 1,
  'HAVE_MKDIRAT': 1,
@@ -606,16 +606,16 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_ZLIB_H': 0,
  'HAVE__GETPTY': 0,
  'HOSTRUNNER': '',
- 'HOST_GNU_TYPE': 'armv7--netbsdelf-eabihf',
+ 'HOST_GNU_TYPE': 'armv6--netbsdelf-eabihf',
  'INCLDIRSTOMAKE': '/usr/pkg/include /usr/pkg/include '
                    '/usr/pkg/include/python3.11 /usr/pkg/include/python3.11',
  'INCLUDEDIR': '/usr/pkg/include',
  'INCLUDEPY': '/usr/pkg/include/python3.11',
- 'INSTALL': '/usr/bin/install -c -o root -g wheel',
- 'INSTALL_DATA': '/usr/bin/install -c -o root -g wheel -m 644',
- 'INSTALL_PROGRAM': '/usr/bin/install -c -s -o root -g wheel -m 755',
- 'INSTALL_SCRIPT': '/usr/bin/install -c -o root -g wheel -m 755',
- 'INSTALL_SHARED': '/usr/bin/install -c -o root -g wheel -m 755',
+ 'INSTALL': '/usr/bin/install -c -o pbulk -g users',
+ 'INSTALL_DATA': '/usr/bin/install -c -o pbulk -g users -m 644',
+ 'INSTALL_PROGRAM': '/usr/bin/install -c -s -o pbulk -g users -m 755',
+ 'INSTALL_SCRIPT': '/usr/bin/install -c -o pbulk -g users -m 755',
+ 'INSTALL_SHARED': '/usr/bin/install -c -o pbulk -g users -m 755',
  'INSTSONAME': 'libpython3.11.so.1.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
@@ -1012,7 +1012,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/Python-3.11.12',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/pbulk/work/lang/python311/work/Python-3.11.13',
  'SCRIPTDIR': '/usr/pkg/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SHELL': '/bin/sh',
@@ -1056,10 +1056,10 @@ build_time_vars = {'ABIFLAGS': '',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/Python-3.11.12 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/pbulk/work/lang/python311/work/Python-3.11.13 '
                './python -E',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/Python-3.11.12 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/pbulk/work/lang/python311/work/Python-3.11.13 '
                './python -E ./Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'ctypes/test \\',
  'TESTTIMEOUT': 1200,
@@ -1090,8 +1090,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/Python-3.11.12',
- 'abs_srcdir': '/mnt/earmv7hf/pkgsrc/lang/python311/work.earmv7hf/Python-3.11.12',
+ 'abs_builddir': '/pbulk/work/lang/python311/work/Python-3.11.13',
+ 'abs_srcdir': '/pbulk/work/lang/python311/work/Python-3.11.13',
  'datarootdir': '/usr/pkg/share',
  'exec_prefix': '/usr/pkg',
  'prefix': '/usr/pkg',
