@@ -3,8 +3,8 @@ build_time_vars = {'ABIFLAGS': '',
  'AC_APPLE_UNIVERSAL_BUILD': 0,
  'AIX_BUILDDATE': 0,
  'AIX_GENUINE_CPLUSPLUS': 0,
- 'ALIGNOF_LONG': 4,
- 'ALIGNOF_SIZE_T': 4,
+ 'ALIGNOF_LONG': 8,
+ 'ALIGNOF_SIZE_T': 8,
  'ALT_SOABI': 0,
  'ANDROID_API_LEVEL': 0,
  'AR': 'ar',
@@ -15,71 +15,66 @@ build_time_vars = {'ABIFLAGS': '',
  'BINDIR': '/usr/pkg/bin',
  'BINLIBDEST': '/usr/pkg/lib/python3.11',
  'BLDLIBRARY': '-L. -lpython3.11',
- 'BLDSHARED': 'gcc -pthread -shared -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
-              '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
-              '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib -Wl,-R/usr/pkg/lib '
-              '-L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
+ 'BLDSHARED': 'gcc -pthread -shared -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib '
+              '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
+              '-Wl,-R/usr/X11R7/lib -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib '
+              '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
+              '-Wl,-R/usr/X11R7/lib',
  'BOOTSTRAP_HEADERS': '\\',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'm68k--netbsdelf',
+ 'BUILD_GNU_TYPE': 'aarch64--netbsd',
  'BYTESTR_DEPS': '\\',
  'CC': 'gcc -pthread',
  'CCSHARED': '-fPIC',
- 'CFLAGS': '-Wsign-compare -DNDEBUG -O2 -fno-reorder-blocks -I/usr/include '
-           '-I/usr/include/readline -I/usr/pkg/include -O2 -fno-reorder-blocks '
-           '-I/usr/include -I/usr/include/readline -I/usr/pkg/include -O2 '
-           '-fno-reorder-blocks -I/usr/include -I/usr/include/readline '
+ 'CFLAGS': '-Wsign-compare -DNDEBUG -O2 -I/usr/include -I/usr/pkg/include -O2 '
+           '-I/usr/include -I/usr/pkg/include -O2 -I/usr/include '
            '-I/usr/pkg/include',
  'CFLAGSFORSHARED': '-fPIC',
  'CFLAGS_ALIASING': '',
  'CFLAGS_NODIST': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
                 'Makefile.pre.in',
- 'CONFIGURE_CFLAGS': '-O2 -fno-reorder-blocks -I/usr/include '
-                     '-I/usr/include/readline -I/usr/pkg/include',
+ 'CONFIGURE_CFLAGS': '-O2 -I/usr/include -I/usr/pkg/include',
  'CONFIGURE_CFLAGS_NODIST': '-std=gnu11 -Wextra -Wno-unused-parameter '
                             '-Wno-missing-field-initializers '
                             '-Wstrict-prototypes '
                             '-Werror=implicit-function-declaration '
                             '-fvisibility=hidden',
- 'CONFIGURE_CPPFLAGS': '-I/usr/include -I/usr/include/readline '
-                       '-I/usr/pkg/include',
- 'CONFIGURE_LDFLAGS': '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+ 'CONFIGURE_CPPFLAGS': '-I/usr/include -I/usr/pkg/include',
+ 'CONFIGURE_LDFLAGS': '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                       '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
                       '-Wl,-R/usr/X11R7/lib',
  'CONFIGURE_LDFLAGS_NODIST': '',
  'CONFIGURE_LDFLAGS_NOLTO': '',
  'CONFIG_ARGS': "'--enable-shared' '--with-openssl=/usr' '--with-system-expat' "
                 "'--with-system-ffi' '--without-ensurepip' "
-                "'--without-pymalloc' '--with-readline=editline' "
-                "'--prefix=/usr/pkg' '--build=m68k--netbsdelf' "
-                "'--host=m68k--netbsdelf' '--mandir=/usr/pkg/man' "
-                "'--enable-option-checking=yes' 'build_alias=m68k--netbsdelf' "
-                "'host_alias=m68k--netbsdelf' "
-                "'PKG_CONFIG=/tmp/pkgbuild/lang/python311/work.m68k/.tools/bin/pkg-config' "
+                "'--without-pymalloc' '--with-readline=readline' "
+                "'--prefix=/usr/pkg' '--build=aarch64--netbsd' "
+                "'--host=aarch64--netbsd' '--mandir=/usr/pkg/man' "
+                "'--enable-option-checking=yes' 'build_alias=aarch64--netbsd' "
+                "'host_alias=aarch64--netbsd' "
+                "'PKG_CONFIG=/pbulk/work/lang/python311/work/.tools/bin/pkg-config' "
                 "'PKG_CONFIG_PATH=' "
-                "'PKG_CONFIG_LIBDIR=/tmp/pkgbuild/lang/python311/work.m68k/.buildlink/lib/pkgconfig:/tmp/pkgbuild/lang/python311/work.m68k/.buildlink/share/pkgconfig' "
-                "'CC=gcc' 'CFLAGS=-O2 -fno-reorder-blocks -I/usr/include "
-                "-I/usr/include/readline -I/usr/pkg/include' "
-                "'LDFLAGS=-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib "
+                "'PKG_CONFIG_LIBDIR=/pbulk/work/lang/python311/work/.buildlink/lib/pkgconfig:/pbulk/work/lang/python311/work/.buildlink/share/pkgconfig' "
+                "'CC=gcc' 'CFLAGS=-O2 -I/usr/include -I/usr/pkg/include' "
+                "'LDFLAGS=-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib "
                 "-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib' "
-                "'LIBS=' 'CPPFLAGS=-I/usr/include -I/usr/include/readline "
-                "-I/usr/pkg/include'",
+                "'LIBS=' 'CPPFLAGS=-I/usr/include -I/usr/pkg/include'",
  'CONFINCLUDEDIR': '/usr/pkg/include',
  'CONFINCLUDEPY': '/usr/pkg/include/python3.11',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/tmp/pkgbuild/lang/python311/work.m68k/Python-3.11.10/coverage.info',
+ 'COVERAGE_INFO': '/pbulk/work/lang/python311/work/Python-3.11.13/coverage.info',
  'COVERAGE_LCOV_OPTIONS': '--rc lcov_branch_coverage=1',
- 'COVERAGE_REPORT': '/tmp/pkgbuild/lang/python311/work.m68k/Python-3.11.10/lcov-report',
+ 'COVERAGE_REPORT': '/pbulk/work/lang/python311/work/Python-3.11.13/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--rc lcov_branch_coverage=1 --branch-coverage '
                             '--title "CPython 3.11 LCOV report [commit $(shell '
                             ')]"',
- 'CPPFLAGS': '-I. -I./Include -I/usr/include -I/usr/include/readline '
-             '-I/usr/pkg/include -I/usr/include -I/usr/include/readline '
+ 'CPPFLAGS': '-I. -I./Include -I/usr/include -I/usr/pkg/include -I/usr/include '
              '-I/usr/pkg/include',
  'CXX': 'c++ -pthread',
- 'DECIMAL_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_32=1 -DANSI=1',
+ 'DECIMAL_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_64=1 -DANSI=1 '
+                   '-DHAVE_UINT128_T=1',
  'DECIMAL_LDFLAGS': '-lm Modules/_decimal/libmpdec/libmpdec.a',
  'DEEPFREEZE_C': 'Python/deepfreeze/deepfreeze.c',
  'DEEPFREEZE_DEPS': './Tools/scripts/deepfreeze.py _bootstrap_python '
@@ -100,8 +95,8 @@ build_time_vars = {'ABIFLAGS': '',
  'DLINCLDIR': '.',
  'DLLLIBRARY': '',
  'DOUBLE_IS_ARM_MIXED_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 1,
- 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 0,
+ 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 0,
+ 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 1,
  'DTRACE': '',
  'DTRACE_DEPS': '\\',
  'DTRACE_HEADERS': '',
@@ -119,7 +114,7 @@ build_time_vars = {'ABIFLAGS': '',
  'EXTRA_CFLAGS': '',
  'EXT_SUFFIX': '.so',
  'FILEMODE': 644,
- 'FLOAT_WORDS_BIGENDIAN': 1,
+ 'FLOAT_WORDS_BIGENDIAN': 0,
  'FREEZE_MODULE': './_bootstrap_python ./Programs/_freeze_module.py',
  'FREEZE_MODULE_BOOTSTRAP': './Programs/_freeze_module',
  'FREEZE_MODULE_BOOTSTRAP_DEPS': 'Programs/_freeze_module',
@@ -251,10 +246,10 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_FUTIMES': 1,
  'HAVE_FUTIMESAT': 0,
  'HAVE_GAI_STRERROR': 1,
- 'HAVE_GCC_ASM_FOR_MC68881': 1,
+ 'HAVE_GCC_ASM_FOR_MC68881': 0,
  'HAVE_GCC_ASM_FOR_X64': 0,
  'HAVE_GCC_ASM_FOR_X87': 0,
- 'HAVE_GCC_UINT128_T': 0,
+ 'HAVE_GCC_UINT128_T': 1,
  'HAVE_GDBM_DASH_NDBM_H': 0,
  'HAVE_GDBM_H': 0,
  'HAVE_GDBM_NDBM_H': 0,
@@ -322,7 +317,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_KILLPG': 1,
  'HAVE_KQUEUE': 1,
  'HAVE_LANGINFO_H': 1,
- 'HAVE_LARGEFILE_SUPPORT': 1,
+ 'HAVE_LARGEFILE_SUPPORT': 0,
  'HAVE_LCHFLAGS': 1,
  'HAVE_LCHMOD': 1,
  'HAVE_LCHOWN': 1,
@@ -430,12 +425,12 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_RECVFROM': 1,
  'HAVE_RENAMEAT': 1,
  'HAVE_RL_APPEND_HISTORY': 1,
- 'HAVE_RL_CATCH_SIGNAL': 0,
- 'HAVE_RL_COMPDISP_FUNC_T': 0,
- 'HAVE_RL_COMPLETION_APPEND_CHARACTER': 0,
+ 'HAVE_RL_CATCH_SIGNAL': 1,
+ 'HAVE_RL_COMPDISP_FUNC_T': 1,
+ 'HAVE_RL_COMPLETION_APPEND_CHARACTER': 1,
  'HAVE_RL_COMPLETION_DISPLAY_MATCHES_HOOK': 1,
  'HAVE_RL_COMPLETION_MATCHES': 1,
- 'HAVE_RL_COMPLETION_SUPPRESS_APPEND': 0,
+ 'HAVE_RL_COMPLETION_SUPPRESS_APPEND': 1,
  'HAVE_RL_PRE_INPUT_HOOK': 1,
  'HAVE_RL_RESIZE_TERMINAL': 1,
  'HAVE_RPC_RPC_H': 1,
@@ -612,7 +607,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_ZLIB_H': 0,
  'HAVE__GETPTY': 0,
  'HOSTRUNNER': '',
- 'HOST_GNU_TYPE': 'm68k--netbsdelf',
+ 'HOST_GNU_TYPE': 'aarch64--netbsd',
  'INCLDIRSTOMAKE': '/usr/pkg/include /usr/pkg/include '
                    '/usr/pkg/include/python3.11 /usr/pkg/include/python3.11',
  'INCLUDEDIR': '/usr/pkg/include',
@@ -626,54 +621,46 @@ build_time_vars = {'ABIFLAGS': '',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
  'LDCXXSHARED': 'c++ -pthread -shared',
- 'LDFLAGS': '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib -Wl,-R/usr/pkg/lib '
-            '-L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib -L/usr/lib -Wl,-R/usr/lib '
-            '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
-            '-Wl,-R/usr/X11R7/lib',
+ 'LDFLAGS': '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+            '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
+            '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+            '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
  'LDFLAGS_NODIST': '',
  'LDLIBRARY': 'libpython3.11.so',
  'LDLIBRARYDIR': '',
- 'LDSHARED': 'gcc -pthread -shared -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
-             '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
-             '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib -Wl,-R/usr/pkg/lib '
-             '-L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
+ 'LDSHARED': 'gcc -pthread -shared -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib '
+             '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
+             '-Wl,-R/usr/X11R7/lib -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib '
+             '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
+             '-Wl,-R/usr/X11R7/lib',
  'LDVERSION': '3.11',
  'LIBC': '',
  'LIBDEST': '/usr/pkg/lib/python3.11',
  'LIBDIR': '/usr/pkg/lib',
  'LIBEXPAT_A': 'Modules/expat/libexpat.a',
- 'LIBEXPAT_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -fno-reorder-blocks '
-                    '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                    '-O2 -fno-reorder-blocks -I/usr/include '
-                    '-I/usr/include/readline -I/usr/pkg/include -O2 '
-                    '-fno-reorder-blocks -I/usr/include '
-                    '-I/usr/include/readline -I/usr/pkg/include -std=gnu11 '
-                    '-Wextra -Wno-unused-parameter '
-                    '-Wno-missing-field-initializers -Wstrict-prototypes '
-                    '-Werror=implicit-function-declaration '
+ 'LIBEXPAT_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -I/usr/include '
+                    '-I/usr/pkg/include -O2 -I/usr/include -I/usr/pkg/include '
+                    '-O2 -I/usr/include -I/usr/pkg/include -std=gnu11 -Wextra '
+                    '-Wno-unused-parameter -Wno-missing-field-initializers '
+                    '-Wstrict-prototypes -Werror=implicit-function-declaration '
                     '-fvisibility=hidden  -I./Include/internal -I. -I./Include '
-                    '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                    '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                    '-fPIC -fPIC',
+                    '-I/usr/include -I/usr/pkg/include -I/usr/include '
+                    '-I/usr/pkg/include -fPIC -fPIC',
  'LIBEXPAT_HEADERS': '\\',
  'LIBEXPAT_OBJS': '\\',
  'LIBFFI_INCLUDEDIR': '/usr/pkg/include',
  'LIBM': '-lm',
  'LIBMPDEC_A': 'Modules/_decimal/libmpdec/libmpdec.a',
- 'LIBMPDEC_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_32=1 -DANSI=1 '
-                    '-Wsign-compare -DNDEBUG -O2 -fno-reorder-blocks '
-                    '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                    '-O2 -fno-reorder-blocks -I/usr/include '
-                    '-I/usr/include/readline -I/usr/pkg/include -O2 '
-                    '-fno-reorder-blocks -I/usr/include '
-                    '-I/usr/include/readline -I/usr/pkg/include -std=gnu11 '
-                    '-Wextra -Wno-unused-parameter '
+ 'LIBMPDEC_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_64=1 -DANSI=1 '
+                    '-DHAVE_UINT128_T=1 -Wsign-compare -DNDEBUG -O2 '
+                    '-I/usr/include -I/usr/pkg/include -O2 -I/usr/include '
+                    '-I/usr/pkg/include -O2 -I/usr/include -I/usr/pkg/include '
+                    '-std=gnu11 -Wextra -Wno-unused-parameter '
                     '-Wno-missing-field-initializers -Wstrict-prototypes '
                     '-Werror=implicit-function-declaration '
                     '-fvisibility=hidden  -I./Include/internal -I. -I./Include '
-                    '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                    '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                    '-fPIC -fPIC',
+                    '-I/usr/include -I/usr/pkg/include -I/usr/include '
+                    '-I/usr/pkg/include -fPIC -fPIC',
  'LIBMPDEC_HEADERS': '\\',
  'LIBMPDEC_OBJS': '\\',
  'LIBOBJDIR': 'Python/',
@@ -819,8 +806,8 @@ build_time_vars = {'ABIFLAGS': '',
  'MODULE__CTYPES_TEST_STATE': 'yes',
  'MODULE__DATETIME_LDFLAGS': '-lm',
  'MODULE__DATETIME_STATE': 'yes',
- 'MODULE__DECIMAL_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_32=1 '
-                           '-DANSI=1',
+ 'MODULE__DECIMAL_CFLAGS': '-I./Modules/_decimal/libmpdec -DCONFIG_64=1 '
+                           '-DANSI=1 -DHAVE_UINT128_T=1',
  'MODULE__DECIMAL_DEPS': './Modules/_decimal/docstrings.h \\ '
                          'Modules/_decimal/libmpdec/libmpdec.a',
  'MODULE__DECIMAL_LDFLAGS': '-lm Modules/_decimal/libmpdec/libmpdec.a',
@@ -843,7 +830,7 @@ build_time_vars = {'ABIFLAGS': '',
  'MODULE__LOCALE_LDFLAGS': '',
  'MODULE__LSPROF_STATE': 'yes',
  'MODULE__LZMA_CFLAGS': '',
- 'MODULE__LZMA_LDFLAGS': '-Wl,-R/usr/lib -llzma',
+ 'MODULE__LZMA_LDFLAGS': '-llzma',
  'MODULE__LZMA_STATE': 'yes',
  'MODULE__MD5_DEPS': './Modules/hashlib.h',
  'MODULE__MD5_STATE': 'yes',
@@ -922,8 +909,7 @@ build_time_vars = {'ABIFLAGS': '',
  'OPENSSL_LDFLAGS': '-L/usr/lib',
  'OPENSSL_LIBS': '-lssl -lcrypto',
  'OPENSSL_RPATH': '',
- 'OPT': '-DNDEBUG -O2 -fno-reorder-blocks -I/usr/include '
-        '-I/usr/include/readline -I/usr/pkg/include',
+ 'OPT': '-DNDEBUG -O2 -I/usr/include -I/usr/pkg/include',
  'OTHER_LIBTOOL_OPT': '',
  'PACKAGE_BUGREPORT': 0,
  'PACKAGE_NAME': 0,
@@ -959,80 +945,65 @@ build_time_vars = {'ABIFLAGS': '',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
  'PY_BUILTIN_HASHLIB_HASHES': '"md5,sha1,sha256,sha512,sha3,blake2"',
- 'PY_BUILTIN_MODULE_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -fno-reorder-blocks '
-                             '-I/usr/include -I/usr/include/readline '
-                             '-I/usr/pkg/include -O2 -fno-reorder-blocks '
-                             '-I/usr/include -I/usr/include/readline '
-                             '-I/usr/pkg/include -O2 -fno-reorder-blocks '
-                             '-I/usr/include -I/usr/include/readline '
+ 'PY_BUILTIN_MODULE_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -I/usr/include '
+                             '-I/usr/pkg/include -O2 -I/usr/include '
+                             '-I/usr/pkg/include -O2 -I/usr/include '
                              '-I/usr/pkg/include -std=gnu11 -Wextra '
                              '-Wno-unused-parameter '
                              '-Wno-missing-field-initializers '
                              '-Wstrict-prototypes '
                              '-Werror=implicit-function-declaration '
                              '-fvisibility=hidden  -I./Include/internal -I. '
-                             '-I./Include -I/usr/include '
-                             '-I/usr/include/readline -I/usr/pkg/include '
-                             '-I/usr/include -I/usr/include/readline '
-                             '-I/usr/pkg/include -fPIC -DPy_BUILD_CORE_BUILTIN',
- 'PY_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -fno-reorder-blocks -I/usr/include '
-              '-I/usr/include/readline -I/usr/pkg/include -O2 '
-              '-fno-reorder-blocks -I/usr/include -I/usr/include/readline '
-              '-I/usr/pkg/include -O2 -fno-reorder-blocks -I/usr/include '
-              '-I/usr/include/readline -I/usr/pkg/include',
+                             '-I./Include -I/usr/include -I/usr/pkg/include '
+                             '-I/usr/include -I/usr/pkg/include -fPIC '
+                             '-DPy_BUILD_CORE_BUILTIN',
+ 'PY_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -I/usr/include -I/usr/pkg/include '
+              '-O2 -I/usr/include -I/usr/pkg/include -O2 -I/usr/include '
+              '-I/usr/pkg/include',
  'PY_CFLAGS_NODIST': '-std=gnu11 -Wextra -Wno-unused-parameter '
                      '-Wno-missing-field-initializers -Wstrict-prototypes '
                      '-Werror=implicit-function-declaration '
                      '-fvisibility=hidden  -I./Include/internal',
  'PY_COERCE_C_LOCALE': 1,
- 'PY_CORE_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -fno-reorder-blocks '
-                   '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                   '-O2 -fno-reorder-blocks -I/usr/include '
-                   '-I/usr/include/readline -I/usr/pkg/include -O2 '
-                   '-fno-reorder-blocks -I/usr/include -I/usr/include/readline '
-                   '-I/usr/pkg/include -std=gnu11 -Wextra '
+ 'PY_CORE_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -I/usr/include '
+                   '-I/usr/pkg/include -O2 -I/usr/include -I/usr/pkg/include '
+                   '-O2 -I/usr/include -I/usr/pkg/include -std=gnu11 -Wextra '
                    '-Wno-unused-parameter -Wno-missing-field-initializers '
                    '-Wstrict-prototypes -Werror=implicit-function-declaration '
                    '-fvisibility=hidden  -I./Include/internal -I. -I./Include '
-                   '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                   '-I/usr/include -I/usr/include/readline -I/usr/pkg/include '
-                   '-fPIC -DPy_BUILD_CORE',
- 'PY_CORE_LDFLAGS': '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+                   '-I/usr/include -I/usr/pkg/include -I/usr/include '
+                   '-I/usr/pkg/include -fPIC -DPy_BUILD_CORE',
+ 'PY_CORE_LDFLAGS': '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                     '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
-                    '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+                    '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                     '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
- 'PY_CPPFLAGS': '-I. -I./Include -I/usr/include -I/usr/include/readline '
-                '-I/usr/pkg/include -I/usr/include -I/usr/include/readline '
-                '-I/usr/pkg/include',
+ 'PY_CPPFLAGS': '-I. -I./Include -I/usr/include -I/usr/pkg/include '
+                '-I/usr/include -I/usr/pkg/include',
  'PY_ENABLE_SHARED': 1,
  'PY_FORMAT_SIZE_T': '"z"',
- 'PY_LDFLAGS': '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib -Wl,-R/usr/pkg/lib '
-               '-L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib -L/usr/lib '
-               '-Wl,-R/usr/lib -L/usr/pkg/lib -Wl,-R/usr/pkg/lib '
-               '-L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
+ 'PY_LDFLAGS': '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+               '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
+               '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+               '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
  'PY_LDFLAGS_NODIST': '',
- 'PY_LDFLAGS_NOLTO': '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+ 'PY_LDFLAGS_NOLTO': '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                      '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
-                     '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+                     '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                      '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
  'PY_SQLITE_ENABLE_LOAD_EXTENSION': 0,
  'PY_SQLITE_HAVE_SERIALIZE': 1,
  'PY_SSL_DEFAULT_CIPHERS': 1,
  'PY_SSL_DEFAULT_CIPHER_STRING': 0,
- 'PY_STDMODULE_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -fno-reorder-blocks '
-                        '-I/usr/include -I/usr/include/readline '
-                        '-I/usr/pkg/include -O2 -fno-reorder-blocks '
-                        '-I/usr/include -I/usr/include/readline '
-                        '-I/usr/pkg/include -O2 -fno-reorder-blocks '
-                        '-I/usr/include -I/usr/include/readline '
+ 'PY_STDMODULE_CFLAGS': '-Wsign-compare -DNDEBUG -O2 -I/usr/include '
+                        '-I/usr/pkg/include -O2 -I/usr/include '
+                        '-I/usr/pkg/include -O2 -I/usr/include '
                         '-I/usr/pkg/include -std=gnu11 -Wextra '
                         '-Wno-unused-parameter -Wno-missing-field-initializers '
                         '-Wstrict-prototypes '
                         '-Werror=implicit-function-declaration '
                         '-fvisibility=hidden  -I./Include/internal -I. '
-                        '-I./Include -I/usr/include -I/usr/include/readline '
-                        '-I/usr/pkg/include -I/usr/include '
-                        '-I/usr/include/readline -I/usr/pkg/include -fPIC',
+                        '-I./Include -I/usr/include -I/usr/pkg/include '
+                        '-I/usr/include -I/usr/pkg/include -fPIC',
  'PY_SUPPORT_TIER': 0,
  'Py_DEBUG': 0,
  'Py_ENABLE_SHARED': 1,
@@ -1044,7 +1015,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/tmp/pkgbuild/lang/python311/work.m68k/Python-3.11.10',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/pbulk/work/lang/python311/work/Python-3.11.13',
  'SCRIPTDIR': '/usr/pkg/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SHELL': '/bin/sh',
@@ -1056,18 +1027,18 @@ build_time_vars = {'ABIFLAGS': '',
  'SIZEOF_FLOAT': 4,
  'SIZEOF_FPOS_T': 264,
  'SIZEOF_INT': 4,
- 'SIZEOF_LONG': 4,
- 'SIZEOF_LONG_DOUBLE': 12,
+ 'SIZEOF_LONG': 8,
+ 'SIZEOF_LONG_DOUBLE': 16,
  'SIZEOF_LONG_LONG': 8,
  'SIZEOF_OFF_T': 8,
  'SIZEOF_PID_T': 4,
  'SIZEOF_PTHREAD_KEY_T': 4,
- 'SIZEOF_PTHREAD_T': 4,
+ 'SIZEOF_PTHREAD_T': 8,
  'SIZEOF_SHORT': 2,
- 'SIZEOF_SIZE_T': 4,
+ 'SIZEOF_SIZE_T': 8,
  'SIZEOF_TIME_T': 8,
- 'SIZEOF_UINTPTR_T': 4,
- 'SIZEOF_VOID_P': 4,
+ 'SIZEOF_UINTPTR_T': 8,
+ 'SIZEOF_VOID_P': 8,
  'SIZEOF_WCHAR_T': 4,
  'SIZEOF__BOOL': 1,
  'SOABI': 'cpython-311',
@@ -1088,10 +1059,10 @@ build_time_vars = {'ABIFLAGS': '',
  'SYS_SELECT_WITH_SYS_TIME': 1,
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/tmp/pkgbuild/lang/python311/work.m68k/Python-3.11.10 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/pbulk/work/lang/python311/work/Python-3.11.13 '
                './python -E',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/tmp/pkgbuild/lang/python311/work.m68k/Python-3.11.10 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/pbulk/work/lang/python311/work/Python-3.11.13 '
                './python -E ./Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'ctypes/test \\',
  'TESTTIMEOUT': 1200,
@@ -1114,16 +1085,16 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_DOC_STRINGS': 1,
  'WITH_DTRACE': 0,
  'WITH_DYLD': 0,
- 'WITH_EDITLINE': 1,
+ 'WITH_EDITLINE': 0,
  'WITH_FREELISTS': 1,
  'WITH_LIBINTL': 1,
  'WITH_NEXT_FRAMEWORK': 0,
  'WITH_PYMALLOC': 0,
  'WITH_VALGRIND': 0,
- 'X87_DOUBLE_ROUNDING': 1,
+ 'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/tmp/pkgbuild/lang/python311/work.m68k/Python-3.11.10',
- 'abs_srcdir': '/tmp/pkgbuild/lang/python311/work.m68k/Python-3.11.10',
+ 'abs_builddir': '/pbulk/work/lang/python311/work/Python-3.11.13',
+ 'abs_srcdir': '/pbulk/work/lang/python311/work/Python-3.11.13',
  'datarootdir': '/usr/pkg/share',
  'exec_prefix': '/usr/pkg',
  'prefix': '/usr/pkg',
